@@ -228,6 +228,19 @@ class QueueViewController: UIViewController {
 
         topConstraint?.constant = topConstraintConstant
         reload()
+
+        guard let currentMedia = playbackService.currentlyPlayingMedia else {
+            return
+        }
+
+        let currentIndex = mediaList.index(of: currentMedia)
+
+        guard currentIndex != NSNotFound else {
+            return
+        }
+
+        let currentIndexPath = IndexPath(row: Int(currentIndex), section: 0)
+        queueCollectionView.scrollToItem(at: currentIndexPath, at: .centeredVertically, animated: true)
     }
 
     @objc init(medialibraryService: MediaLibraryService) {
